@@ -1,8 +1,21 @@
-/***********************************************************************
- *
- *  kftabdlg.cpp
- *
- **********************************************************************/
+/*******************************************************************
+* kftabdlg.cpp
+* 
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License as
+* published by the Free Software Foundation; either version 2 of 
+* the License, or (at your option) any later version.
+* 
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* GNU General Public License for more details.
+* 
+* You should have received a copy of the GNU General Public License
+* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* 
+******************************************************************/
+
 #include "kftabdlg.h"
 
 #include <QtGui/QButtonGroup>
@@ -12,8 +25,6 @@
 #include <QtGui/QCheckBox>
 #include <QtGui/QWhatsThis>
 
-#include <QtGui/QComboBox>
-#include <QtGui/QSpinBox>
 #include <QtGui/QPushButton>
 #include <QtGui/QApplication>
 #include <QtGui/QDesktopWidget>
@@ -21,17 +32,22 @@
 #include <kcalendarsystem.h>
 #include <kglobal.h>
 #include <kcombobox.h>
+#include <kurlcombobox.h>
+#include <kurlcompletion.h>
 #include <klineedit.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <KNumInput>
 #include <kfiledialog.h>
 #include <kregexpeditorinterface.h>
 #include <kservicetypetrader.h>
 #include <kparts/componentfactory.h>
 #include <kstandarddirs.h>
+
+#include "kdatecombo.h"
 #include "kquery.h"
 // Static utility functions
-static void save_pattern(QComboBox *, const QString &, const QString &);
+static void save_pattern(KComboBox *, const QString &, const QString &);
 
 #define SPECIAL_TYPES 7
 
@@ -44,7 +60,7 @@ struct LessMimeType_ByComment
 };
 
 KfindTabWidget::KfindTabWidget(QWidget *parent)
-  : QTabWidget( parent ), regExpDialog(0)
+  : KTabWidget( parent ), regExpDialog(0)
 {
     // This validator will be used for all numeric edit fields
     //KDigitValidator *digitV = new KDigitValidator(this);
@@ -57,12 +73,14 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     nameBox = new KComboBox(pages[0]);
     nameBox->setEditable( true );
     nameBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);  // allow smaller than widest entry
-    QLabel * namedL = new QLabel(i18n("&Named:"), pages[0]);
+    QLabel * namedL = new QLabel(i18nc("this is the label for the name textfield","&Named:"), pages[0]);
     namedL->setBuddy( nameBox );
     namedL->setObjectName( "named" );
     namedL->setToolTip( i18n("You can use wildcard matching and \";\" for separating multiple names") );
-    dirBox  = new KComboBox(pages[0]);
+    dirBox  = new KUrlComboBox(KUrlComboBox::Directories, pages[0]);
     dirBox->setEditable( true );
+    dirBox->setCompletionObject(new KUrlCompletion(KUrlCompletion::DirCompletion));
+    dirBox->setAutoDeleteCompletionObject(true);
     dirBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);  // allow smaller than widest entry
     QLabel * lookinL = new QLabel(i18n("Look &in:"), pages[0]);
     lookinL->setBuddy( dirBox );
@@ -71,12 +89,13 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     caseSensCb  = new QCheckBox(i18n("Case s&ensitive search"), pages[0]);
     browseB    = new QPushButton(i18n("&Browse..."), pages[0]);
     useLocateCb = new QCheckBox(i18n("&Use files index"), pages[0]);
-
+    hiddenFilesCb = new QCheckBox(i18n("Show &hidden files"), pages[0]);
     // Setup
 
     subdirsCb->setChecked(true);
     caseSensCb->setChecked(false);
     useLocateCb->setChecked(false);
+    hiddenFilesCb->setChecked(false);
     if(KStandardDirs::findExe("locate").isEmpty())
         useLocateCb->setEnabled(false);
 
@@ -84,8 +103,8 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     nameBox->setFocus();
     dirBox->setDuplicatesEnabled(false);
 
-    nameBox->setInsertPolicy(QComboBox::AtTop);
-    dirBox->setInsertPolicy(QComboBox::AtTop);
+    nameBox->setInsertPolicy(QComboBox::InsertAtTop);
+    dirBox->setInsertPolicy(QComboBox::InsertAtTop);
 
     const QString nameWhatsThis
       = i18n("<qt>Enter the filename you are looking for. <br />"
@@ -120,7 +139,7 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     QGridLayout *grid = new QGridLayout( pages[0] );
     grid->setMargin( KDialog::marginHint() );
     grid->setSpacing( KDialog::spacingHint() );
-    QBoxLayout *subgrid = new QVBoxLayout();
+    QVBoxLayout *subgrid = new QVBoxLayout();
     grid->addWidget( namedL, 0, 0 );
     grid->addWidget( nameBox, 0, 1, 1, 3 );
     grid->addWidget( lookinL, 1, 0 );
@@ -128,9 +147,18 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     grid->addWidget( browseB, 1, 2);
     grid->setColumnStretch(1,1);
     grid->addLayout( subgrid, 2, 1, 1, 2 );
-    subgrid->addWidget( subdirsCb );
-    subgrid->addWidget( caseSensCb);
-    subgrid->addWidget( useLocateCb );
+    
+    QHBoxLayout * layoutOne = new QHBoxLayout();
+    layoutOne->addWidget( subdirsCb );
+    layoutOne->addWidget( hiddenFilesCb );
+    
+    QHBoxLayout * layoutTwo = new QHBoxLayout();
+    layoutTwo->addWidget( caseSensCb);
+    layoutTwo->addWidget( useLocateCb );
+    
+    subgrid->addLayout( layoutOne );
+    subgrid->addLayout( layoutTwo );
+    
     subgrid->addStretch(1);
 
     // Signals
@@ -149,19 +177,14 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     findCreated =  new QCheckBox(i18n("Find all files created or &modified:"), pages[1]);
     bg  = new QButtonGroup();
     rb[0] = new QRadioButton(i18n("&between"), pages[1] );
-    rb[1] = new QRadioButton(i18n("&during the previous"), pages[1] );
+    rb[1] = new QRadioButton(pages[1]); // text set in updateDateLabels
     andL = new QLabel(i18n("and"), pages[1]);
     andL->setObjectName( "and" );
     betweenType = new KComboBox( pages[1] );
-    betweenType->setEditable( false );
     betweenType->setObjectName( "comboBetweenType" );
-    betweenType->addItem(i18n("minute(s)"));
-    betweenType->addItem(i18n("hour(s)"));
-    betweenType->addItem(i18n("day(s)"));
-    betweenType->addItem(i18n("month(s)"));
-    betweenType->addItem(i18n("year(s)"));
+    betweenType->addItems(QVector<QString>(5).toList()); // texts set in updateDateLabels
     betweenType->setCurrentIndex(1);
-
+    updateDateLabels(1, 1);
 
     QDate dt = KGlobal::locale()->calendar()->addYears(QDate::currentDate(), -1);
 
@@ -169,23 +192,21 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     fromDate->setObjectName( "fromDate" );
     toDate = new KDateCombo(pages[1] );
     toDate->setObjectName( "toDate" );
-    timeBox = new QSpinBox( pages[1] );
+    timeBox = new KIntSpinBox( pages[1] );
     timeBox->setRange( 1, 60 );
     timeBox->setSingleStep( 1 );
     timeBox->setObjectName( "timeBox" );
 
     sizeBox =new KComboBox( pages[1] );
-    sizeBox->setEditable( false );
     sizeBox->setObjectName( "sizeBox" );
     QLabel * sizeL   =new QLabel(i18n("File &size is:"), pages[1]);
     sizeL->setBuddy( sizeBox );
-    sizeEdit=new QSpinBox(pages[1] );
+    sizeEdit=new KIntSpinBox(pages[1] );
     sizeEdit->setRange( 0, INT_MAX );
     sizeEdit->setSingleStep( 1 );
     sizeEdit->setObjectName( "sizeEdit" );
     sizeEdit->setValue(1);
     sizeUnitBox =new KComboBox( pages[1] );
-    sizeUnitBox->setEditable( false );
     sizeUnitBox->setObjectName( "sizeUnitBox" );
 
     m_usernameBox = new KComboBox( pages[1] );
@@ -199,12 +220,12 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     QLabel *groupLabel= new QLabel(i18n("Owned by &group:"),pages[1]);
     groupLabel->setBuddy( m_groupBox );
 
-    sizeBox ->addItem( i18n("(none)") );
+    sizeBox ->addItem( i18nc("file size isn't considered in the search","(none)") );
     sizeBox ->addItem( i18n("At Least") );
     sizeBox ->addItem( i18n("At Most") );
     sizeBox ->addItem( i18n("Equal To") );
 
-    sizeUnitBox ->addItem( i18n("Bytes") );
+    sizeUnitBox ->addItem( i18np("Byte", "Bytes", 1) );
     sizeUnitBox ->addItem( i18n("KiB") );
     sizeUnitBox ->addItem( i18n("MiB") );
     sizeUnitBox ->addItem( i18n("GiB") );
@@ -215,8 +236,8 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
 
     m_usernameBox->setDuplicatesEnabled(false);
     m_groupBox->setDuplicatesEnabled(false);
-    m_usernameBox->setInsertPolicy(QComboBox::AtTop);
-    m_groupBox->setInsertPolicy(QComboBox::AtTop);
+    m_usernameBox->setInsertPolicy(QComboBox::InsertAtTop);
+    m_groupBox->setInsertPolicy(QComboBox::InsertAtTop);
 
 
     // Setup
@@ -261,6 +282,9 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     connect( findCreated, SIGNAL(toggled(bool)),  SLOT(fixLayout()) );
     connect( bg, SIGNAL(buttonClicked(QAbstractButton*)), this,  SLOT(fixLayout()) );
     connect( sizeBox, SIGNAL(activated(int)), this, SLOT(slotSizeBoxChanged(int)));
+    connect( timeBox, SIGNAL(valueChanged(int)), this, SLOT(slotUpdateDateLabelsForNumber(int)));
+    connect( betweenType, SIGNAL(currentIndexChanged(int)), this, SLOT(slotUpdateDateLabelsForType(int)));
+    connect( sizeEdit, SIGNAL(valueChanged(int)), this, SLOT(slotUpdateByteComboBox(int)));
 
 
     // ************ Page Three
@@ -269,10 +293,9 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     pages[2]->setObjectName( "page3" );
 
     typeBox =new KComboBox( pages[2] );
-    typeBox->setEditable( false );
     typeBox->setObjectName( "typeBox" );
     typeBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);  // allow smaller than widest entry
-    QLabel * typeL   =new QLabel( i18n("File &type:"), pages[2] );
+    QLabel * typeL   =new QLabel( i18nc("label for the file type combobox","File &type:"), pages[2] );
     typeL->setBuddy( typeBox );
     textEdit=new KLineEdit(pages[2]);
     textEdit->setClearButtonShown(true);
@@ -377,7 +400,7 @@ KfindTabWidget::KfindTabWidget(QWidget *parent)
     }
 
     addTab( pages[0], i18n("Name/&Location") );
-    addTab( pages[2], i18n("C&ontents") );
+    addTab( pages[2], i18nc("tab name: search by contents","C&ontents") );
     addTab( pages[1], i18n("&Properties") );
 
 
@@ -453,11 +476,11 @@ void KfindTabWidget::setURL( const KUrl & url )
 void KfindTabWidget::initMimeTypes()
 {
     KMimeType::List sortedList;
-    foreach ( KMimeType::Ptr type, KMimeType::allMimeTypes() )
+    foreach ( const KMimeType::Ptr &type, KMimeType::allMimeTypes() )
     {
       if ( (!type->comment().isEmpty())
-           && (!type->name().startsWith("kdedevice/"))
-           && (!type->name().startsWith("all/")) )
+           && (!type->name().startsWith( QString("kdedevice/") ))
+           && (!type->name().startsWith( QString("all/") )) )
         sortedList.append(type);
     }
     qSort( sortedList.begin(), sortedList.end(), LessMimeType_ByComment() );
@@ -473,11 +496,11 @@ void KfindTabWidget::initSpecialMimeTypes()
 	  const KMimeType* type = (*it).data();
 
       if(!type->comment().isEmpty()) {
-        if(type->name().startsWith("image/"))
+        if(type->name().startsWith( QString("image/") ))
            m_ImageTypes.append(type->name());
-        else if(type->name().startsWith("video/"))
+        else if(type->name().startsWith( QString("video/") ))
           m_VideoTypes.append(type->name());
-        else if(type->name().startsWith("audio/"))
+        else if(type->name().startsWith( QString("audio/") ))
           m_AudioTypes.append(type->name());
       }
     }
@@ -531,7 +554,7 @@ void KfindTabWidget::loadHistory()
 void KfindTabWidget::slotEditRegExp()
 {
   if ( ! regExpDialog )
-    regExpDialog = KServiceTypeTrader::createInstanceFromQuery<QDialog>( "KRegExpEditor/KRegExpEditor", QString(), this );
+    regExpDialog = KServiceTypeTrader::createInstanceFromQuery<KDialog>( "KRegExpEditor/KRegExpEditor", QString(), this );
 
   KRegExpEditorInterface *iface = qobject_cast<KRegExpEditorInterface *>( regExpDialog );
   if ( !iface )
@@ -601,7 +624,7 @@ bool KfindTabWidget::isDateValid()
   else if ( QDate::currentDate() < hi1 )
     str = i18n("Unable to search dates in the future.");
 
-  if (!str.isNull()) {
+  if (!str.isEmpty()) {
     KMessageBox::sorry(0, str);
     return false;
   }
@@ -758,6 +781,8 @@ void KfindTabWidget::setQuery(KQuery *query)
   //Use locate to speed-up search ?
   query->setUseFileIndex(useLocateCb->isChecked());
 
+  query->setShowHiddenFiles(hiddenFilesCb->isChecked());
+  
   query->setContext(textEdit->text(), caseContextCb->isChecked(),
   	binaryContextCb->isChecked(), regexpContentCb->isChecked());
 }
@@ -816,12 +841,12 @@ void KfindTabWidget::fixLayout()
     toDate->setEnabled(false);
     andL->setEnabled(false);
     timeBox->setEnabled(false);
-    for(i=0; i<2; i++)
+    for(i=0; i<2; ++i)
       rb[i]->setEnabled(false);
     betweenType->setEnabled(false);
   }
   else {
-    for(i=0; i<2; i++)
+    for(i=0; i<2; ++i)
       rb[i]->setEnabled(true);
 
     fromDate->setEnabled(rb[0]->isChecked());
@@ -841,6 +866,34 @@ bool KfindTabWidget::isSearchRecursive()
   return subdirsCb->isChecked();
 }
 
+void KfindTabWidget::slotUpdateDateLabelsForNumber(int value)
+{
+  updateDateLabels(betweenType->currentIndex(), value);
+}
+
+void KfindTabWidget::slotUpdateDateLabelsForType(int index)
+{
+  updateDateLabels(index, timeBox->value());
+}
+
+void KfindTabWidget::updateDateLabels(int type, int value)
+{
+  QString typeKey(type == 0 ? 'i' : type == 1 ? 'h' : type == 2 ? 'd' : type == 3 ? 'm' : 'y');
+  rb[1]->setText(ki18ncp("during the previous minute(s)/hour(s)/...; "
+                         "dynamic context 'type': 'i' minutes, 'h' hours, 'd' days, 'm' months, 'y' years",
+                         "&during the previous", "&during the previous").subs(value).inContext("type", typeKey).toString());
+  betweenType->setItemText(0, i18ncp("use date ranges to search files by modified time", "minute", "minutes", value));
+  betweenType->setItemText(1, i18ncp("use date ranges to search files by modified time", "hour", "hours", value));
+  betweenType->setItemText(2, i18ncp("use date ranges to search files by modified time", "day", "days", value));
+  betweenType->setItemText(3, i18ncp("use date ranges to search files by modified time", "month", "months", value));
+  betweenType->setItemText(4, i18ncp("use date ranges to search files by modified time", "year", "years", value));
+}
+
+void KfindTabWidget::slotUpdateByteComboBox(int value)
+{
+  sizeUnitBox->setItemText(0, i18np("Byte", "Bytes", value) );
+}
+
 
 /**
    Digit validator. Allows only digits to be typed.
@@ -870,7 +923,7 @@ QValidator::State KDigitValidator::validate( QString & input, int & ) const
 //*******************************************************
 //             Static utility functions
 //*******************************************************
-static void save_pattern(QComboBox *obj,
+static void save_pattern(KComboBox *obj,
 			 const QString & group, const QString & entry)
 {
   // QComboBox allows insertion of items more than specified by
@@ -898,7 +951,7 @@ QSize KfindTabWidget::sizeHint() const
   // Like in minicli, we changed the combobox size policy so that they can resize down,
   // and then we simply provide a reasonable size hint for the whole window, depending
   // on the screen width.
-  QSize sz = QTabWidget::sizeHint();
+  QSize sz = KTabWidget::sizeHint();
   KfindTabWidget* me = const_cast<KfindTabWidget*>( this );
   const int screenWidth = qApp->desktop()->screenGeometry(me).width();
   if ( sz.width() > screenWidth / 2 )
