@@ -55,7 +55,7 @@ class KBookmarkGroup;
 class KBookmarkMenu;
 class KBookmarkActionMenu;
 class KCMultiDialog;
-class KNewMenu;
+class KNewFileMenu;
 class KToggleAction;
 class KonqBidiHistoryAction;
 class KBookmarkBar;
@@ -63,7 +63,7 @@ class KonqView;
 class KonqFrameContainerBase;
 class KonqFrameContainer;
 class KToolBarPopupAction;
-class KAnimatedButton;
+class KonqAnimatedLogo;
 class KonqViewManager;
 class ToggleViewGUIClient;
 class KonqMainWindowIface;
@@ -72,6 +72,7 @@ class KonqRun;
 class KConfigGroup;
 class KUrlRequester;
 class KBookmarkManager;
+class KonqHistoryDialog;
 struct HistoryEntry;
 
 namespace KParts {
@@ -89,7 +90,6 @@ class KONQ_TESTS_EXPORT KonqMainWindow : public KParts::MainWindow, public KonqF
 {
   Q_OBJECT
   Q_PROPERTY( int viewCount READ viewCount )
-  Q_PROPERTY( int activeViewsCount READ activeViewsCount )
   Q_PROPERTY( int linkableViewsCount READ linkableViewsCount )
   Q_PROPERTY( QString locationBarURL READ locationBarURL )
   Q_PROPERTY( bool fullScreenMode READ fullScreenMode )
@@ -170,9 +170,6 @@ public:
   // Total number of views
   int viewCount() const { return m_mapViews.count(); }
 
-    // Number of views not in "passive" mode
-    int activeViewsCount() const;
-
     // Number of views not in "passive" mode and not locked
     int activeViewsNotLockedCount() const;
 
@@ -182,9 +179,6 @@ public:
     // Number of main views (non-toggle non-passive views)
     int mainViewsCount() const;
 
-    // Return true if we are showing a view that supports this mimeType.
-    bool hasViewWithMimeType(const QString& mimeType) const;
-
   typedef QMap<KParts::ReadOnlyPart *, KonqView *> MapViews;
 
   const MapViews & viewMap() const { return m_mapViews; }
@@ -256,8 +250,8 @@ public:
 
   static QList<KonqMainWindow*> *mainWindowList() { return s_lstViews; }
 
-  // public for konq_guiclients
-  void viewCountChanged();
+    void linkableViewCountChanged();
+    void viewCountChanged();
 
     /**
      * For the view manager: we are loading the profile from this config file,
@@ -305,8 +299,10 @@ public:
 
   // KonqFrameContainerBase implementation END
 
-  KonqFrameBase* workingTab()const { return m_pWorkingTab; }
-  void setWorkingTab( KonqFrameBase* tab ) { m_pWorkingTab = tab; }
+    /**
+     * When using RMB on a tab, remember the tab we are showing a popup for.
+     */
+    void setWorkingTab(int index);
 
   static bool isMimeTypeAssociatedWithSelf( const QString &mimeType );
   static bool isMimeTypeAssociatedWithSelf( const QString &mimeType, const KService::Ptr &offer );
@@ -328,7 +324,10 @@ public:
   QString currentProfile() const;
     void applyWindowSizeFromProfile(const KConfigGroup& profileGroup);
 
-  void updateHistoryActions();
+    void updateHistoryActions();
+
+    // Public for unit tests
+    void prepareForPopupMenu(const KFileItemList &items, const KParts::OpenUrlArguments &args, const KParts::BrowserArguments& browserArgs);
 
 Q_SIGNALS:
   void viewAdded( KonqView *view );
@@ -415,7 +414,7 @@ public Q_SLOTS:
   void slotSplitViewHorizontal();
   void slotSplitViewVertical();
   void slotRemoveOtherTabs();
-  void slotRemoveOtherTabsPopupDelayed();
+  void slotRemoveTabPopup();
 
 private Q_SLOTS:
   void slotViewCompleted( KonqView * view );
@@ -429,7 +428,7 @@ private Q_SLOTS:
 
   void slotBreakOffTab();
   void slotBreakOffTabPopup();
-  void slotBreakOffTabPopupDelayed();
+  void breakOffTab(int);
 
   void slotPopupNewWindow();
   void slotPopupThisWindow();
@@ -442,8 +441,8 @@ private Q_SLOTS:
   void slotReloadPopup();
   void slotReloadAllTabs();
   void slotRemoveTab();
-  void slotRemoveTabPopup();
-  void slotRemoveTabPopupDelayed();
+  void removeTab(int tabIndex);
+  void removeOtherTabs(int tabIndex);
 
   void slotActivateNextTab();
   void slotActivatePrevTab();
@@ -455,10 +454,7 @@ private Q_SLOTS:
 
     void slotOpenEmbedded(KService::Ptr);
 
-  // Connected to KSycoca
-  void slotDatabaseChanged();
-
-  // Connected to KApp
+  // Connected to KGlobalSettings
   void slotReconfigure();
 
   void slotForceSaveMainWindowSettings();
@@ -484,6 +480,7 @@ private Q_SLOTS:
   void slotUpActivated(QAction* action);
   void slotBackActivated(QAction* action);
   void slotForwardActivated(QAction* action);
+  void slotHomePopupActivated(QAction* action);
   void slotGoHistoryDelayed();
 
   void slotCompletionModeChanged( KGlobalSettings::Completion );
@@ -617,14 +614,10 @@ private:
 
   QObject* lastFrame( KonqView *view );
 
-    // Maximum height of the animated logo qtoolbutton (m_paAnimatedLogo)
-    int maxThrobberHeight();
-    void setAnimatedLogoSize();
-
 private: // members
   KonqUndoManager* m_pUndoManager;
 
-  KNewMenu * m_pMenuNew;
+  KNewFileMenu * m_pMenuNew;
 
   KAction *m_paPrint;
 
@@ -633,6 +626,7 @@ private: // members
   KToolBarPopupAction *m_paUp;
   KToolBarPopupAction *m_paBack;
   KToolBarPopupAction *m_paForward;
+  KToolBarPopupAction *m_paHomePopup;
   /// Action for the trash that contains closed tabs/windows
   KToolBarPopupAction *m_paClosedItems;
   KActionMenu *m_paSessions;
@@ -674,7 +668,7 @@ private: // members
   KAction *m_paConfigureExtensions;
   KAction *m_paConfigureSpellChecking;
 
-  KAnimatedButton *m_paAnimatedLogo;
+  KonqAnimatedLogo *m_paAnimatedLogo;
 
   KBookmarkBar *m_paBookmarkBar;
 
@@ -687,6 +681,7 @@ private: // members
 
   KToggleFullScreenAction *m_ptaFullScreen;
 
+  bool m_fullyConstructed:1;
   bool m_bLocationBarConnected:1;
   bool m_bURLEnterLock:1;
   // Global settings
@@ -695,7 +690,6 @@ private: // members
   bool m_bNeedApplyKonqMainWindowSettings:1;
   bool m_urlCompletionStarted:1;
   bool m_prevMenuBarVisible:1;
-  bool m_fullyConstructed:1;
 
   int m_goBuffer;
   Qt::MouseButtons m_goMouseState;
@@ -713,7 +707,8 @@ private: // members
   KonqViewManager *m_pViewManager;
   KonqFrameBase* m_pChildFrame;
 
-  KonqFrameBase* m_pWorkingTab;
+    // RMB on a tab: we need to store the tab number until the slots are called
+    int m_workingTab;
 
     // Store a number of things when opening a popup, they are needed
     // in the slots connected to the popup's actions.
@@ -748,13 +743,11 @@ private: // members
 
   QString m_currentDir; // stores current dir for relative URLs whenever applicable
 
+  QPointer<KonqHistoryDialog> m_historyDialog;
+
     // TODO: move to a KonqPreloadHandler class
   static bool s_preloaded;
   static KonqMainWindow* s_preloadedWindow;
-
-public:
-
-  static QFile *s_crashlog_file;
 };
 
 #endif // KONQMAINWINDOW_H
