@@ -261,10 +261,10 @@ KonqView* KonqViewManager::addTabFromHistory( KonqView* currentView, int steps,
 }
 
 
-void KonqViewManager::duplicateTab( KonqFrameBase* currentFrame, bool openAfterCurrentPage )
+void KonqViewManager::duplicateTab(int tabIndex, bool openAfterCurrentPage)
 {
 #ifdef DEBUG_VIEWMGR
-  kDebug() << currentFrame;
+  kDebug() << tabIndex;
   m_pMainWindow->dumpViewList();
   printFullHierarchy();
 #endif
@@ -274,11 +274,12 @@ void KonqViewManager::duplicateTab( KonqFrameBase* currentFrame, bool openAfterC
   KConfig config( tempFile.fileName() );
   KConfigGroup profileGroup( &config, "Profile" );
 
-  QString prefix = KonqFrameBase::frameTypeToString(currentFrame->frameType()) + QString::number(0);
+  KonqFrameBase* tab = tabContainer()->tabAt(tabIndex);
+  QString prefix = KonqFrameBase::frameTypeToString(tab->frameType()) + QString::number(0); // always T0
   profileGroup.writeEntry( "RootItem", prefix );
   prefix.append( QLatin1Char( '_' ) );
   KonqFrameBase::Options flags = KonqFrameBase::saveHistoryItems;
-  currentFrame->saveConfig( profileGroup, prefix, flags, 0L, 0, 1);
+  tab->saveConfig( profileGroup, prefix, flags, 0L, 0, 1);
 
   loadRootItem( profileGroup, tabContainer(), KUrl(), true, KUrl(), openAfterCurrentPage );
 
@@ -293,12 +294,10 @@ void KonqViewManager::duplicateTab( KonqFrameBase* currentFrame, bool openAfterC
 #endif
 }
 
-KonqMainWindow* KonqViewManager::breakOffTab( KonqFrameBase* currentFrame, const QSize& windowSize )
+KonqMainWindow* KonqViewManager::breakOffTab(int tab, const QSize& windowSize)
 {
-    Q_ASSERT(currentFrame);
-
 #ifdef DEBUG_VIEWMGR
-  kDebug() << currentFrame;
+    kDebug() << "tab=" << tab;
   m_pMainWindow->dumpViewList();
   printFullHierarchy();
 #endif
@@ -308,11 +307,12 @@ KonqMainWindow* KonqViewManager::breakOffTab( KonqFrameBase* currentFrame, const
   KSharedConfigPtr config = KSharedConfig::openConfig( tempFile.fileName() );
   KConfigGroup profileGroup( config, "Profile" );
 
-  QString prefix = KonqFrameBase::frameTypeToString(currentFrame->frameType()) + QString::number(0);
+  KonqFrameBase* tabFrame = tabContainer()->tabAt(tab);
+  QString prefix = KonqFrameBase::frameTypeToString(tabFrame->frameType()) + QString::number(0); // always T0
   profileGroup.writeEntry( "RootItem", prefix );
   prefix.append( QLatin1Char( '_' ) );
   KonqFrameBase::Options flags = KonqFrameBase::saveHistoryItems;
-  currentFrame->saveConfig( profileGroup, prefix, flags, 0L, 0, 1);
+  tabFrame->saveConfig( profileGroup, prefix, flags, 0L, 0, 1);
 
   KonqMainWindow *mainWindow = new KonqMainWindow(KUrl(), m_pMainWindow->xmlFile());
 
@@ -320,7 +320,7 @@ KonqMainWindow* KonqViewManager::breakOffTab( KonqFrameBase* currentFrame, const
   mainWindow->viewManager()->loadRootItem( profileGroup, newTabContainer, KUrl(), true, KUrl() );
   mainWindow->viewManager()->setCurrentProfile( currentProfile() );
 
-  removeTab( currentFrame, false );
+  removeTab( tabFrame, false );
 
   mainWindow->enableAllActions( true );
   mainWindow->resize( windowSize );
@@ -394,8 +394,8 @@ void KonqViewManager::removeTab( KonqFrameBase* currentFrame, bool emitAboutToRe
   printFullHierarchy();
 #endif
 
-  if ( m_tabContainer->count() == 1 ) // TODO: see #214378
-    return;
+  if ( m_tabContainer->count() == 1 )
+    m_pMainWindow->slotAddTab(); // #214378
 
   if(emitAboutToRemoveSignal)
     emit aboutToRemoveTab(currentFrame);
@@ -508,6 +508,14 @@ void KonqViewManager::showTab( KonqView *view )
   }
 }
 
+void KonqViewManager::showTab(int tabIndex)
+{
+    if (m_tabContainer->currentIndex() != tabIndex) {
+        m_tabContainer->setCurrentIndex(tabIndex);
+        emitActivePartChanged();
+    }
+}
+
 void KonqViewManager::updatePixmaps()
 {
     const QList<KonqView*> viewList = KonqViewCollector::collect(tabContainer());
@@ -708,44 +716,44 @@ void KonqViewManager::clear()
 KonqView *KonqViewManager::chooseNextView(KonqView *view)
 {
     //kDebug() << view;
-    const KonqMainWindow::MapViews& mapViews = m_pMainWindow->viewMap();
 
-    KonqMainWindow::MapViews::const_iterator it = mapViews.begin();
-    const KonqMainWindow::MapViews::const_iterator end = mapViews.end();
-    if (view) { // find it in the map - can't use the key since view->part() might be 0L
-        while (it != end && it.value() != view)
-            ++it;
+    int it = 0;
+    const QList<KonqView*> viewList = KonqViewCollector::collect(m_pMainWindow);
+    if (viewList.isEmpty()) {
+        return 0; // We have no view at all - this used to happen with totally-empty-profiles
+    }
+
+    if (view) { // find it in the list
+        it = viewList.indexOf(view);
     }
 
     // the view should always be in the list
-    if (it == end) {
-        if (view)
-            kWarning() << "View" << view << "is not in list!" ;
-        it = mapViews.begin();
-        if (it == end)
-            return 0; // We have no view at all - this used to happen with totally-empty-profiles
+    if (it == -1) {
+        kWarning() << "View" << view << "is not in list!" ;
+        it = 0;
     }
 
     bool rewinded = false;
-    KonqMainWindow::MapViews::const_iterator startIt = it;
+    const int startIndex = it;
+    const int end = viewList.count();
 
-    //kDebug() << "count=" << mapViews.count();
+    //kDebug() << "count=" << end;
     while (true) {
-        //kDebug() << "*KonqViewManager::chooseNextView going next";
+        //kDebug() << "going next";
         if (++it == end) { // move to next
             // end reached: restart from begin (but only once)
             if (!rewinded) {
-                it = mapViews.begin();
+                it = 0;
                 rewinded = true;
             } else {
                 break; // nothing found, probably buggy profile
             }
         }
 
-        if (it == startIt && view)
+        if (it == startIndex && view)
             break; // no next view found
 
-        KonqView *nextView = it.value();
+        KonqView *nextView = viewList.at(it);;
         if (nextView && !nextView->isPassiveMode())
             return nextView;
         //kDebug() << "nextView=" << nextView << "passive=" << nextView->isPassiveMode();
@@ -855,7 +863,7 @@ KonqView *KonqViewManager::setupView( KonqFrameContainerBase *parentContainer,
 void KonqViewManager::saveViewProfileToFile(const QString & fileName, const QString & profileName, KonqFrameBase::Options options)
 {
     const QString path = KStandardDirs::locateLocal("data", QString::fromLatin1("konqueror/profiles/") +
-                                                    fileName, KGlobal::mainComponent());
+                                                    fileName);
     QFile::remove(path); // in case it exists already
 
     KConfig _cfg(path, KConfig::SimpleConfig);
@@ -997,8 +1005,10 @@ void KonqViewManager::loadViewProfileFromGroup( const KConfigGroup &profileGroup
       // TODO choose a linked view if any (instead of just the first one),
       // then open the same URL in any non-linked one
     } else {
-        if (m_pMainWindow->locationBarURL().isEmpty()) // No URL -> the user will want to type one
+        if (forcedUrl.isEmpty() && m_pMainWindow->locationBarURL().isEmpty()) {
+            // No URL -> the user will want to type one
             m_pMainWindow->focusLocationBar();
+        }
     }
 
     // Window size
@@ -1018,37 +1028,22 @@ void KonqViewManager::loadViewProfileFromGroup( const KConfigGroup &profileGroup
 
 void KonqViewManager::setActivePart(KParts::Part *part, QWidget *)
 {
-    doSetActivePart( part );
+    doSetActivePart( static_cast<KParts::ReadOnlyPart*>(part) );
 }
 
-void KonqViewManager::doSetActivePart( KParts::Part *part )
+void KonqViewManager::doSetActivePart( KParts::ReadOnlyPart *part )
 {
-    //kDebug() << part;
-    //if ( part )
-    //    kDebug() << part->metaObject()->className() << part->name();
+    if (part)
+        kDebug() << part << part->url();
 
     KParts::Part* mainWindowActivePart = m_pMainWindow->currentView()
                                          ? m_pMainWindow->currentView()->part() : 0;
     if (part == activePart() && mainWindowActivePart == part)
     {
-      //if (part)
-      //    kDebug() << "Part is already active!";
+      //kDebug() << "Part is already active!";
       return;
     }
 
-    // Don't activate when part changed in non-active tab
-    KonqView* partView = m_pMainWindow->childView((KParts::ReadOnlyPart*)part);
-    if (partView)
-    {
-      KonqFrameContainerBase* parentContainer = partView->frame()->parentContainer();
-      if (parentContainer->frameType() == KonqFrameBase::Tabs)
-      {
-        KonqFrameTabs* parentFrameTabs = static_cast<KonqFrameTabs*>(parentContainer);
-        if (partView->frame() != parentFrameTabs->currentWidget())
-           return;
-      }
-    }
-
     if (m_pMainWindow && m_pMainWindow->currentView())
       m_pMainWindow->currentView()->setLocationBarURL(m_pMainWindow->locationBarURL());
 
@@ -1394,30 +1389,23 @@ void KonqViewManager::slotProfileActivated(QAction* action)
                                                  "LoadProfileTabsConfirm" ) == KMessageBox::Cancel )
             return;
     }
-    KonqView *originalView = m_pMainWindow->currentView();
-    bool showTabCalled = false;
+    const int originalTabIndex = m_tabContainer->currentIndex();
     foreach ( KonqFrameBase* frame, m_tabContainer->childFrameList() )
     {
         KonqView *view = frame->activeChildView();
-        if (view && view->part() && (view->part()->metaObject()->indexOfProperty("modified") != -1)) {
-            QVariant prop = view->part()->property("modified");
-            if (prop.isValid() && prop.toBool()) {
+        if (view && view->isModified()) {
                 showTab( view );
-                showTabCalled = true;
                 if ( KMessageBox::warningContinueCancel( 0,
                                                          i18n("This tab contains changes that have not been submitted.\nLoading a profile will discard these changes."),
                                                          i18n("Discard Changes?"), KGuiItem(i18n("&Discard Changes")), KStandardGuiItem::cancel(), "discardchangesloadprofile") != KMessageBox::Continue )
                     /* WE: maybe KStandardGuiItem(Discard) here? */
                 {
-                    showTab( originalView );
+                    showTab(originalTabIndex);
                     return;
                 }
-            }
         }
     }
-    if ( showTabCalled && originalView )
-        showTab( originalView );
-
+    showTab(originalTabIndex);
 
     const QString profilePath = action->data().toString();
     const QString fileName = KUrl(profilePath).fileName();
