@@ -286,6 +286,7 @@ TerminalDisplay::TerminalDisplay(QWidget *parent)
 ,_fontHeight(1)
 ,_fontWidth(1)
 ,_fontAscent(1)
+,_boldIntense(true)
 ,_lines(1)
 ,_columns(1)
 ,_usedLines(1)
@@ -369,7 +370,7 @@ TerminalDisplay::TerminalDisplay(QWidget *parent)
   setAttribute(Qt::WA_OpaquePaintEvent);
 
   _gridLayout = new QGridLayout(this);
-  _gridLayout->setMargin(0);
+  _gridLayout->setContentsMargins(0, 0, 0, 0);
 
   setLayout( _gridLayout ); 
 
@@ -516,7 +517,7 @@ void TerminalDisplay::drawLineCharString(    QPainter& painter, int x, int y, co
 {
         const QPen& currentPen = painter.pen();
         
-        if ( attributes->rendition & RE_BOLD )
+        if ( (attributes->rendition & RE_BOLD) && _boldIntense )
         {
             QPen boldPen(currentPen);
             boldPen.setWidth(3);
@@ -672,9 +673,9 @@ void TerminalDisplay::drawCharacters(QPainter& painter,
    
     // setup bold and underline
     bool useBold;
-    ColorEntry::FontWeight weight = style->fontWeight(_colorTable);
+    ColorEntry::FontWeight weight = style->fontWeight(_colorTable);    
     if (weight == ColorEntry::UseCurrentFormat)
-        useBold = style->rendition & RE_BOLD || font().bold();
+        useBold = ((style->rendition & RE_BOLD) && _boldIntense) || font().bold();
     else
         useBold = (weight == ColorEntry::Bold) ? true : false;
     bool useUnderline = style->rendition & RE_UNDERLINE || font().underline();
@@ -1918,7 +1919,7 @@ void TerminalDisplay::extendSelection( const QPoint& position )
   if ( oldpos.y() < textBounds.top() )
   {
       linesBeyondWidget = (textBounds.top()-oldpos.y()) / _fontHeight;
-    _scrollBar->setValue(_scrollBar->value()-linesBeyondWidget-1); // scrollback
+    _scrollBar->setValue(_scrollBar->value()-linesBeyondWidget-1); // history
   }
 
   int charColumn = 0;
@@ -2888,7 +2889,7 @@ void TerminalDisplay::outputSuspended(bool suspended)
             _outputSuspendedLabel->setAutoFillBackground(true);
             _outputSuspendedLabel->setBackgroundRole(QPalette::Base);
             _outputSuspendedLabel->setFont(QApplication::font());
-            _outputSuspendedLabel->setMargin(5);
+            _outputSuspendedLabel->setContentsMargins(5, 5, 5, 5);
 
             //enable activation of "Xon/Xoff" link in label
             _outputSuspendedLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse | 
