@@ -151,13 +151,13 @@ void ViewManager::setupActions()
         KAction* splitLeftRightAction = new KAction( KIcon("view-split-left-right"),
                                                       i18nc("@action:inmenu", "Split View Left/Right"),
                                                       this );
-        splitLeftRightAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_L) );
+        splitLeftRightAction->setShortcut( QKeySequence(Qt::CTRL+Qt::Key_ParenLeft) );
         collection->addAction("split-view-left-right",splitLeftRightAction);
         connect( splitLeftRightAction , SIGNAL(triggered()) , this , SLOT(splitLeftRight()) );
 
         KAction* splitTopBottomAction = new KAction( KIcon("view-split-top-bottom") , 
                                              i18nc("@action:inmenu", "Split View Top/Bottom"),this);
-        splitTopBottomAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_T) );
+        splitTopBottomAction->setShortcut( QKeySequence(Qt::CTRL+Qt::Key_ParenRight) );
         collection->addAction("split-view-top-bottom",splitTopBottomAction);
         connect( splitTopBottomAction , SIGNAL(triggered()) , this , SLOT(splitTopBottom()));
 
@@ -167,8 +167,8 @@ void ViewManager::setupActions()
         closeActiveAction->setEnabled(false);
         collection->addAction("close-active-view",closeActiveAction);
         connect( closeActiveAction , SIGNAL(triggered()) , this , SLOT(closeActiveView()) );
-      
-        multiViewOnlyActions << closeActiveAction; 
+
+        multiViewOnlyActions << closeActiveAction;
 
         KAction* closeOtherAction = new KAction( i18nc("@action:inmenu Close Other Views", "Close Others") , this );
         closeOtherAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_O) );
@@ -179,15 +179,15 @@ void ViewManager::setupActions()
         multiViewOnlyActions << closeOtherAction;
 
         KAction* detachViewAction = collection->addAction("detach-view");
-        detachViewAction->setIcon( KIcon("tab-detach") );
-        detachViewAction->setText( i18n("&Detach View") );
+        detachViewAction->setIcon(KIcon("tab-detach"));
+        detachViewAction->setText(i18n("D&etach Current Tab"));
         // Ctrl+Shift+D is not used as a shortcut by default because it is too close
         // to Ctrl+D - which will terminate the session in many cases
-        detachViewAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_H) );
-       
-          connect( this , SIGNAL(splitViewToggle(bool)) , this , SLOT(updateDetachViewState()) ); 
+        detachViewAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_H));
+
+        connect( this , SIGNAL(splitViewToggle(bool)) , this , SLOT(updateDetachViewState()) );
         connect( detachViewAction , SIGNAL(triggered()) , this , SLOT(detachActiveView()) );
-   
+
         // Expand & Shrink Active View
         KAction* expandActiveAction = new KAction( i18nc("@action:inmenu", "Expand View") , this );
         expandActiveAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_BracketRight) );
@@ -314,22 +314,30 @@ void ViewManager::previousView()
 
     container->activatePreviousView();
 }
+
 void ViewManager::detachActiveView()
 {
     // find the currently active view and remove it from its container 
     ViewContainer* container = _viewSplitter->activeContainer();
-    TerminalDisplay* activeView = dynamic_cast<TerminalDisplay*>(container->activeView());
 
-    if (!activeView)
+    detachView(container, container->activeView());
+}
+
+void ViewManager::detachView(ViewContainer* container, QWidget* widgetView)
+{
+    TerminalDisplay * viewToDetach = 
+        dynamic_cast<TerminalDisplay*>(widgetView);
+
+    if (!viewToDetach)
         return;
 
-    emit viewDetached(_sessionMap[activeView]);
+    emit viewDetached(_sessionMap[viewToDetach]);
     
-    _sessionMap.remove(activeView);
+    _sessionMap.remove(viewToDetach);
 
     // remove the view from this window
-    container->removeView(activeView);
-    activeView->deleteLater();
+    container->removeView(viewToDetach);
+    viewToDetach->deleteLater();
 
     // if the container from which the view was removed is now empty then it can be deleted,
     // unless it is the only container in the window, in which case it is left empty
@@ -351,13 +359,8 @@ void ViewManager::sessionFinished()
 
     Session* session = qobject_cast<Session*>(sender());
 
-    if ( _sessionMap[qobject_cast<TerminalDisplay*>(activeView())] == session )
-    {
-        // switch to the next view before deleting the session views to prevent flicker 
-        // occurring as a result of an interval between removing the active view and switching
-        // to the next view
-        nextView();
-    }
+    // We're using setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab)
+    // so no need to manually select next tab.
 
     Q_ASSERT(session);
 
@@ -426,7 +429,8 @@ void ViewManager::splitView(Qt::Orientation orientation)
     {
         Session* session = _sessionMap[(TerminalDisplay*)existingViewIter.next()];
         TerminalDisplay* display = createTerminalDisplay(session);
-        applyProfile(display,SessionManager::instance()->sessionProfile(session),false);
+        const Profile::Ptr info = SessionManager::instance()->sessionProfile(session);
+        applyProfile(display, info, false);
         ViewProperties* properties = createController(session,display);
 
         _sessionMap[display] = session;
@@ -434,8 +438,15 @@ void ViewManager::splitView(Qt::Orientation orientation)
         // create a container using settings from the first 
         // session in the previous container
         if ( !container )
-            container = createContainer(SessionManager::instance()->sessionProfile(session));
+            container = createContainer(info);
 
+        int tabBarMode = info->property<int>(Profile::TabBarMode);
+        if ( tabBarMode == Profile::AlwaysHideTabBar )
+            container->setNavigationDisplayMode(ViewContainer::AlwaysHideNavigation);
+        else if ( tabBarMode == Profile::AlwaysShowTabBar )
+            container->setNavigationDisplayMode(ViewContainer::AlwaysShowNavigation);
+        else if ( tabBarMode == Profile::ShowTabBarAsNeeded )
+            container->setNavigationDisplayMode(ViewContainer::ShowNavigationAsNeeded);
         container->addView(display,properties);
         session->addView( display );
     }
@@ -614,7 +625,16 @@ ViewContainer* ViewManager::createContainer(const Profile::Ptr info)
     switch ( _navigationMethod )
     {
         case TabbedNavigation:    
-            container = new TabbedViewContainer(position,_viewSplitter);
+            {
+                container =
+                    new TabbedViewContainer(position,_viewSplitter);
+
+                connect(container,
+                    SIGNAL(detachTab(ViewContainer*, QWidget*)),
+                    this,
+                    SLOT(detachView(ViewContainer*, QWidget*))
+                    );
+            }
             break;
         case NoNavigation:
         default:
@@ -646,6 +666,7 @@ void ViewManager::containerMoveViewRequest(int index, int id, bool& moved)
         return;
 
     createView(controller->session(),container,index);
+    controller->session()->refresh();
     moved = true;
 }
 void ViewManager::setNavigationMethod(NavigationMethod method)
@@ -806,6 +827,7 @@ void ViewManager::applyProfile(TerminalDisplay* view , const Profile::Ptr info,
   
     // load font 
     view->setAntialias(info->property<bool>(Profile::AntiAliasFonts));
+    view->setBoldIntense(info->property<bool>(Profile::BoldIntense));
     view->setVTFont(info->font());
 
     // set scroll-bar position
