@@ -57,16 +57,12 @@
 #include <Plasma/BusyWidget>
 #include <Plasma/FrameSvg>
 #include <Plasma/Theme>
-
-#ifdef Q_WS_X11
-#  include <QX11Info>
-#  include <X11/Xlib.h>
-#endif
+#include <Plasma/WindowEffects>
 
 
 QTime PopupView::s_lastOpenClose;
 
-PopupView::PopupView(const KUrl &url, const QPoint &pos,
+PopupView::PopupView(const QModelIndex &index, const QPoint &pos,
                      const bool &showPreview, const QStringList &previewPlugins,
                      const IconView *parentView)
     : QWidget(0, Qt::X11BypassWindowManagerHint),
@@ -76,7 +72,6 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos,
       m_iconView(0),
       m_dirModel(0),
       m_model(0),
-      m_url(url),
       m_actionCollection(this),
       m_newMenu(0),
       m_itemActions(0),
@@ -85,6 +80,11 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos,
       m_previewPlugins(previewPlugins)
 {
     setAttribute(Qt::WA_TranslucentBackground);
+#ifdef Q_WS_X11
+    if (KWindowSystem::compositingActive()) {
+        setAttribute(Qt::WA_NoSystemBackground, false);
+    }
+#endif
 
 #ifdef Q_WS_WIN
     setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::Tool);
@@ -99,6 +99,14 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos,
     pal.setColor(QPalette::Text, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
     setPalette(pal);
 
+    KFileItem item = static_cast<const ProxyModel*>(index.model())->itemForIndex(index);
+    if (item.isDesktopFile()) {
+        KDesktopFile file(item.localPath());
+        m_url = file.readUrl();
+    } else {
+        m_url = item.targetUrl();
+    }
+
     m_background = new Plasma::FrameSvg(this);
     m_background->setImagePath("widgets/tooltip");
 
@@ -128,10 +136,7 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos,
         pt.ry() = available.top();
     }
 
-#ifdef Q_WS_X11
-    Atom atom = XInternAtom(QX11Info::display(), "_KDE_SHADOW_OVERRIDE", False);
-    XChangeProperty(QX11Info::display(), winId(), atom, atom, 32, PropModeReplace, 0, 0);
-#endif
+    Plasma::WindowEffects::overrideShadow(winId(), true);
 
     move(pt);
     show();
@@ -185,6 +190,7 @@ void PopupView::init()
     m_model = new ProxyModel(this);
     m_model->setSourceModel(m_dirModel);
     m_model->setSortLocaleAware(true);
+    m_model->setDynamicSortFilter(true);
     m_model->setFilterCaseSensitivity(Qt::CaseInsensitive);
     m_model->setParseDesktopFiles(m_url.protocol() == "desktop");
     m_model->sort(KDirModel::Name, Qt::AscendingOrder);
@@ -203,6 +209,7 @@ void PopupView::init()
     m_iconView->setGridSize(m_parentView->gridSize());
     m_iconView->setWordWrap(m_parentView->wordWrap());
     m_iconView->setIconsMoveable(false);
+    m_iconView->setClickToViewFolders(false);
 
     connect(m_iconView, SIGNAL(activated(QModelIndex)), SLOT(activated(QModelIndex)));
     connect(m_iconView, SIGNAL(contextMenuRequest(QWidget*,QPoint)), SLOT(contextMenuRequest(QWidget*,QPoint)));
@@ -269,7 +276,7 @@ void PopupView::createActions()
     connect(del, SIGNAL(triggered()), SLOT(deleteSelectedIcons()));
 
     // Create the new menu
-    m_newMenu = new KNewMenu(&m_actionCollection, this, "new_menu");
+    m_newMenu = new KNewFileMenu(&m_actionCollection, "new_menu", this);
     connect(m_newMenu->menu(), SIGNAL(aboutToShow()), this, SLOT(aboutToShowCreateNew()));
 
     m_actionCollection.addAction("undo", undo);
@@ -300,8 +307,10 @@ void PopupView::contextMenuRequest(QWidget *widget, const QPoint &screenPos)
 
     foreach (const QModelIndex &index, m_selectionModel->selectedIndexes()) {
         KFileItem item = m_model->itemForIndex(index);
-        hasRemoteFiles |= item.localPath().isEmpty();
-        items.append(item);
+        if (!item.isNull()) {
+            hasRemoteFiles |= item.localPath().isEmpty();
+            items.append(item);
+        }
     }
 
     // Check if we're showing the menu for the trash link
@@ -437,14 +446,22 @@ void PopupView::activated(const QModelIndex &index)
 
 void PopupView::setBusy(bool busy)
 {
+    m_busy = busy;
     if (busy && !m_busyWidget) {
+        QTimer::singleShot(100, this, SLOT(createBusyWidgetIfNeeded()));
+    } else {
+        delete m_busyWidget;
+        m_busyWidget = 0;
+    }
+}
+
+void PopupView::createBusyWidgetIfNeeded()
+{
+    if (m_busy && !m_busyWidget) {
         const int size = qMin(width(), height()) * .3;
         m_busyWidget = new Plasma::BusyWidget;
         m_busyWidget->setGeometry(QStyle::alignedRect(layoutDirection(), Qt::AlignCenter, QSize(size, size), contentsRect()));
         m_scene->addItem(m_busyWidget);
-    } else {
-        delete m_busyWidget;
-        m_busyWidget = 0;
     }
 }
 
@@ -463,7 +480,7 @@ void PopupView::undoTextChanged(const QString &text)
 void PopupView::aboutToShowCreateNew()
 {
     if (m_newMenu) {
-        m_newMenu->slotCheckUpToDate();
+        m_newMenu->checkUpToDate();
         m_newMenu->setPopupFiles(m_url);
     }
 }
@@ -477,11 +494,12 @@ void PopupView::resizeEvent(QResizeEvent *event)
     if (m_view) {
         m_view->setGeometry(contentsRect());
     }
-#ifdef Q_WS_X11
-    if (!QX11Info::isCompositingManagerRunning()) {
+
+    if (KWindowSystem::compositingActive()) {
+        Plasma::WindowEffects::enableBlurBehind(winId(), true, m_background->mask());
+    } else {
         setMask(m_background->mask());
     }
-#endif
 }
 
 void PopupView::paintEvent(QPaintEvent *event)
@@ -506,7 +524,7 @@ void PopupView::contextMenuEvent(QContextMenuEvent *event)
     }
 
     KFileItem rootItem = m_model->itemForIndex(QModelIndex());
-    //The root item is invalid (non-existant)
+    //The root item is invalid (non-existent)
     if (rootItem.isNull()) {
         return;
     }
